/**************************************************************************//**
 * @file        dualtimer.c
 * @version     V1.00
 * $Revision:   0 $
 * $Date:       $
 * @brief       TIMER Driver source file
 *
 * @note
 * Copyright (C) 2016 AMICCOM Electronics Corp. All rights reserved.
 *****************************************************************************/

#include "AMICCOM_CM0.h"

/** @addtogroup Std_Driver Standard Driver
    @{
*/
/** @addtogroup DUALTIMER_Driver DualTimer Driver
    @{
*/

/**
 *
 * @brief   Initial Dualtimer set to free-run mode. Value will set to 0xFFFFFFFF when value count down to 0. 
 *
 * @param   *dualtimer              DualTimer Pointer
 * @param   prescal                 0=/1, 1=/16, 2=/256
 * @param   size                    0=16bits, 1=32bits
 * @return  none
 */
void DualTimer_Freerun_Initial(DualTIMER_Type *dualtimer, uint32_t prescal, uint32_t size)
{
    uint32_t control = ((prescal&0x3ul)<<DualTIMER_CTRL_PRESCALE_Pos) |
                       ((size&0x1ul)<<DualTIMER_CTRL_SIZE_Pos);
    dualtimer->CTRL = control;
}

/**
 *
 * @brief   Initial Dualtimer set to one-shot mode. DualTimer will stop when value count down to 0. 
 *
 * @param   *dualtimer              DualTimer Pointer
 * @param   prescal                 0=/1, 1=/16, 2=/256
 * @param   size                    0=16bits, 1=32bits
 * @param   load                    the value set to 0=16bits, 1=32bits
 * @return  none
 */
void DualTimer_OneShot_Initial(DualTIMER_Type *dualtimer, uint32_t prescal, uint32_t size, uint32_t load)
{
    uint32_t control =  DualTIMER_CTRL_ONESHOOT_Msk |
                        ((prescal&0x3ul)<<DualTIMER_CTRL_PRESCALE_Pos) |
                        ((size&0x1ul)<<DualTIMER_CTRL_SIZE_Pos);
    dualtimer->CTRL = control;
    dualtimer->LOAD = load;
}

/**
 *
 * @brief   Initial Dualtimer set to period mode. When value count down to 0, the value will set to load and continue count. 
 *
 * @param   *dualtimer              DualTimer Pointer
 * @param   prescal                 0=/1, 1=/16, 2=/256
 * @param   size                    0=16bits, 1=32bits
 * @return  none
 */
void DualTimer_Period_Initial(DualTIMER_Type *dualtimer, uint32_t prescal, uint32_t size, uint32_t load)
{
    uint32_t control =  DualTIMER_CTRL_MODE_Msk | 
                        ((prescal&0x3ul)<<DualTIMER_CTRL_PRESCALE_Pos) |
                        ((size&0x1ul)<<DualTIMER_CTRL_SIZE_Pos);
    dualtimer->CTRL = control;
    dualtimer->LOAD = load;
}

/**
 *
 * @brief   Enable the microcontroller dualtimer interrupts.
 *
 * @param   *dualtimer              DualTimer Pointer
 * @return  none
 */
void DualTimer_EnableIRQ(DualTIMER_Type *dualtimer)
{
    dualtimer->CTRL |= DualTIMER_CTRL_INTEN_Msk;
}

/**
 *
 * @brief   Disable the microcontroller dualtimer interrutps.
 *
 * @param   *dualtimer              DualTimer Pointer
 * @return  none
 */
void DualTimer_DisableIRQ(DualTIMER_Type *dualtimer)
{
    dualtimer->CTRL &= ~DualTIMER_CTRL_INTEN_Msk;
}

/**
 *
 * @brief   Clears the dualtimer IRQ if set.
 *
 * @param   *dualtimer              DualTimer Pointer
 * @return  none
 */
void DualTimer_ClearIRQ(DualTIMER_Type *dualtimer)
{
    dualtimer->INTCLR = 1;
}


/**
 *
 * @brief   Start the DualTimer.
 *
 * @param   *dualtimer              DualTimer Pointer
 * @return  none
 */
void DualTimer_StartTimer(DualTIMER_Type *dualtimer)
{
    dualtimer->CTRL |= DualTIMER_CTRL_EN_Msk;
}

/**
 *
 * @brief   Stop the DualTimer.
 *
 * @param   *dualtimer              DualTimer Pointer
 * @return  none
 */
void DualTimer_StopTimer(DualTIMER_Type *dualtimer)
{
    dualtimer->CTRL &= ~DualTIMER_CTRL_EN_Msk;
}

/**
 *
 * @brief   Returns the current value of the dualtimer.
 *
 * @param   *dualtimer              DualTimer Pointer
 * @return  dualtimer current value
 */
uint32_t DualTimer_GetValue(DualTIMER_Type *dualtimer)
{
    return dualtimer->VALUE;
}

/**
 *
 * @brief   Sets the load value of the dualtimer to the specified value. This value also set to current value immediately.
 *
 * @param   *dualtimer              DualTimer Pointer
 * @param   value                   Value to be loaded
 * @return  none
 */
void DualTimer_SetLoad(DualTIMER_Type *dualtimer, uint32_t value)
{
    dualtimer->LOAD = value;
}

/**
 *
 * @brief   Returns the load value of the dualtimer.
 *
 * @param   *dualtimer              DualTimer Pointer
 * @return  load value
 */
uint32_t DualTimer_GetLoad(DualTIMER_Type *dualtimer)
{
    return dualtimer->LOAD;
}

/**
 *
 * @brief   Sets the BGLoad value of the timer to the specified value. This value also set to Load immediately but not set to current value.
 *
 * @param   *dualtimer              DualTimer Pointer
 * @param   value                   Value to be loaded
 * @return  none
 */
void DualTimer_SetBGLoad(DualTIMER_Type *dualtimer, uint32_t value)
{
    dualtimer->BGLOAD = value;
}

/**
 *
 * @brief   Returns the BGLoad value of the dualtimer.
 *
 * @param   *dualtimer              DualTimer Pointer
 * @return  load value
 */
uint32_t DualTimer_GetBGLoad(DualTIMER_Type *dualtimer)
{
    return dualtimer->BGLOAD;
}

/*@}*/ /* end of group DUALTIMER_Driver */
/*@}*/ /* end of group Std_Driver */

